==> beagle-2.1.5.tar.gz: beagle-2.1.5/beagle/GA/src/FloatVector.cpp <==
/*
 *  Open BEAGLE
 *  Copyright (C) 2001-2004 by Christian Gagne and Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   beagle/GA/src/FloatVector.cpp
 *  \brief  Source code of class GA::FloatVector.
 *  \author Christian Gagne
 *  \author Marc Parizeau
 *  $Revision: 1.8 $
 *  $Date: 2004/07/01 23:58:59 $
 */

#include "beagle/GA.hpp"

#include <algorithm>
#include <cfloat>
#include <float.h>    // To get FLT_MAX on Borland C++ Builder

using namespace Beagle;


/*!
 *  \brief Construct a GA float vector.
 *  \param inSize Size of the vector.
 *  \param inModel Default values of the floats.
 */
GA::FloatVector::FloatVector(unsigned int inSize, float inModel) :
  std::vector<float>(inSize, inModel)
{ }


/*!
 *  \brief Return size (length) of genotype.
 *  \return Size (length) of genotype.
 */
unsigned int GA::FloatVector::getSize() const
{
  return size();
}


/*!
 *  \brief  Test if two float vectors are equals.
 *  \param  inRightObj Second float vector used for the comparison.
 *  \return True if float vectors are equal, false if not.
 *  \par Note:
 *    Returns true if the range [ 0, min(end(),inRightObj.end()) )
 *    are identical when compared element-by-element, and otherwise returns false.
 */
bool GA::FloatVector::isEqual(const Object& inRightObj) const
{
  const GA::FloatVector& lRightFV = castObjectT<const GA::FloatVector&>(inRightObj);
  unsigned int lSizeCompared = minOf<unsigned int>(size(), lRightFV.size());
  return std::equal(begin(), begin()+lSizeCompared, lRightFV.begin());
}


/*!
 *  \brief  Test the ordering of two float vectors.
 *  \param  inRightObj Second float vector used for the ordering test.
 *  \return True if the actual float vector is before the second, false if not.
 *  \par Note:
 *    Returns true if the actual float vector is lexicographically less than the right object
 *    in the range [ 0, min(end(),inRightObj.end()) ) when compared element-by-element,
 *    and otherwise returns false.
 */
bool GA::FloatVector::isLess(const Object& inRightObj) const
{
  const GA::FloatVector& lRightFV = castObjectT<const GA::FloatVector&>(inRightObj);
  unsigned int lSizeCompared = minOf<unsigned int>(size(),lRightFV.size());
  return std::lexicographical_compare(begin(), begin()+lSizeCompared,
                                      lRightFV.begin(), lRightFV.begin()+lSizeCompared);
}


/*!
 *  \brief Read a float vector from an input stream.
 *  \param inNode XML node from which the float vector is read.
 *  \param ioContext Evolutionary context to use to read the float vector.
 *  \throw IOException If a reading error occured (generally bad file format)!
 */
void GA::FloatVector::readWithContext(XMLNode::Handle& inNode, Context& ioContext)
{
  if((inNode->getType() != XMLNode::eTag) || (inNode->getTagName() != "Genotype"))
    throw Beagle_IOExceptionNodeM(*inNode, "tag <Genotype> expected!");

  std::string lType = inNode->getTagAttribute("type");
  if(lType.empty())
    throw Beagle_IOExceptionNodeM(*inNode, "float vector type of the genotype is not present!");
  if(lType != "floatvector")
    throw Beagle_IOExceptionNodeM(*inNode, "type of genotype mismatch, expected \"floatvector\"!");

  XMLNode::Handle lChild = inNode->getFirstChild();
  if((lChild==NULL) || (lChild->getType()!=XMLNode::eString))
    throw Beagle_IOExceptionNodeM(*lChild, "expected content for the float vector!");

  clear();
  std::istringstream lISS(lChild->getValue());
  do {
    float lValue=FLT_MAX;
    lISS >> lValue;
    if(lValue != FLT_MAX) push_back(lValue);
  } while(lISS.get() == int('/'));
}


/*!
 *  \brief Write a float vector into a XML streamer.
 *  \param ioStreamer XML streamer into which the float vector is written.
 */
void GA::FloatVector::write(XMLStreamer& ioStreamer) const
{
  ioStreamer.openTag("Genotype");
  ioStreamer.insertAttribute("type", "floatvector");
  ioStreamer.insertAttribute("size", uint2str(size()));
  std::ostringstream lOSS;
  for(size_type i=0; i<size(); i++) {
    lOSS << (*this)[i];
    if(i!=(size()-1)) lOSS << '/';
  }
  ioStreamer.insertString(lOSS.str());
  ioStreamer.closeTag(false);
}

==> beagle-2.2.0.tar.gz: beagle-2.2.0/beagle/GA/src/FloatVector.cpp <==
/*
 *  Open BEAGLE
 *  Copyright (C) 2001-2004 by Christian Gagne and Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   beagle/GA/src/FloatVector.cpp
 *  \brief  Source code of class GA::FloatVector.
 *  \author Christian Gagne
 *  \author Marc Parizeau
 *  $Revision: 1.10 $
 *  $Date: 2004/10/26 16:53:59 $
 */

#include "beagle/GA.hpp"

#include <algorithm>
#include <cfloat>
#include <float.h>    // To get FLT_MAX on Borland C++ Builder

using namespace Beagle;


/*!
 *  \brief Construct a GA float vector.
 *  \param inSize Size of the vector.
 *  \param inModel Default values of the floats.
 */
GA::FloatVector::FloatVector(unsigned int inSize, float inModel) :
  std::vector<double>(inSize, inModel)
{ }


/*!
 *  \brief Return size (length) of genotype.
 *  \return Size (length) of genotype.
 */
unsigned int GA::FloatVector::getSize() const
{
  return size();
}


/*!
 *  \brief  Test if two float vectors are equals.
 *  \param  inRightObj Second float vector used for the comparison.
 *  \return True if float vectors are equal, false if not.
 *  \par Note:
 *    Returns true if the range [ 0, min(end(),inRightObj.end()) )
 *    are identical when compared element-by-element, and otherwise returns false.
 */
bool GA::FloatVector::isEqual(const Object& inRightObj) const
{
  const GA::FloatVector& lRightVector = castObjectT<const GA::FloatVector&>(inRightObj);
  if(size()!=lRightVector.size()) return false;
  const std::vector<double>& lLeft = *this;
  const std::vector<double>& lRight = lRightVector;
  return (lLeft == lRight);
}


/*!
 *  \brief  Test the ordering of two float vectors.
 *  \param  inRightObj Second float vector used for the ordering test.
 *  \return True if the actual float vector is before the second, false if not.
 *  \par Note:
 *    Returns true if the actual float vector is lexicographically less than the right object
 *    in the range [ 0, min(end(),inRightObj.end()) ) when compared element-by-element,
 *    and otherwise returns false.
 */
bool GA::FloatVector::isLess(const Object& inRightObj) const
{
  const GA::FloatVector& lRightVector = castObjectT<const GA::FloatVector&>(inRightObj);
  if(size()!=lRightVector.size()) return false;
  const std::vector<double>& lLeft = *this;
  const std::vector<double>& lRight = lRightVector;
  return (lLeft < lRight);
}


/*!
 *  \brief Read a float vector from an input stream.
 *  \param inNode XML node from which the float vector is read.
 *  \param ioContext Evolutionary context to use to read the float vector.
 *  \throw Beagle::IOException If a reading error occured (generally bad file format)!
 */
void GA::FloatVector::readWithContext(XMLNode::Handle& inNode, Context& ioContext)
{
  if((inNode->getType() != XMLNode::eTag) || (inNode->getTagName() != "Genotype"))
    throw Beagle_IOExceptionNodeM(*inNode, "tag <Genotype> expected!");

  std::string lType = inNode->getTagAttribute("type");
  if(lType.empty())
    throw Beagle_IOExceptionNodeM(*inNode, "float vector type of the genotype is not present!");
  if(lType != "floatvector")
    throw Beagle_IOExceptionNodeM(*inNode, "type of genotype mismatch, expected \"floatvector\"!");

  XMLNode::Handle lChild = inNode->getFirstChild();
  if((lChild==NULL) || (lChild->getType()!=XMLNode::eString))
    throw Beagle_IOExceptionNodeM(*lChild, "expected content for the float vector!");

  resize(0);
  std::istringstream lISS(lChild->getValue());
  while(lISS.good()) {
    double lValue=DBL_MAX;
    lISS >> lValue;
    push_back(lValue);
    if(lISS.good()==false) break;
    int lDelim=lISS.get();
    if((lISS.good()==false) || (lDelim==-1)) break;
  }
}


/*!
 *  \brief Write a float vector into a XML streamer.
 *  \param ioStreamer XML streamer into which the float vector is written.
 */
void GA::FloatVector::write(XMLStreamer& ioStreamer) const
{
  ioStreamer.openTag("Genotype");
  ioStreamer.insertAttribute("type", "floatvector");
  ioStreamer.insertAttribute("size", uint2str(size()));
  std::ostringstream lOSS;
  for(size_type i=0; i<size(); ++i) {
    lOSS << (*this)[i];
    if(i!=(size()-1)) lOSS << ',';
  }
  ioStreamer.insertString(lOSS.str());
  ioStreamer.closeTag(false);
}
